@@ -23,6 +23,7 @@
  */
 
 #include "private-lib-core.h"
+#include "private-lib-async-dns.h"
 
 #if defined(LWS_WITH_CLIENT)
 static int
@@ -119,6 +120,7 @@ __lws_reset_wsi(struct lws *wsi)
 	}
 #endif
 	wsi->retry = 0;
+	wsi->mount_hit = 0;
 
 #if defined(LWS_WITH_CLIENT)
 	lws_dll2_remove(&wsi->dll2_cli_txn_queue);
@@ -136,6 +138,13 @@ __lws_reset_wsi(struct lws *wsi)
 		lws_buflist_destroy_all_segments(&wsi->http.buflist_post_body);
 #endif
 
+#if defined(LWS_WITH_HTTP_DIGEST_AUTH)
+	if (wsi->http.digest_auth_hdr) {
+		lws_free(wsi->http.digest_auth_hdr);
+		wsi->http.digest_auth_hdr = NULL;
+	}
+#endif
+
 #if defined(LWS_WITH_SERVER)
 	lws_dll2_remove(&wsi->listen_list);
 #endif
@@ -212,6 +221,9 @@ __lws_free_wsi(struct lws *wsi)
 
 	lws_context_assert_lock_held(wsi->a.context);
 
+	/* just in case */
+	lws_dll2_remove(&wsi->pre_natal);
+
 #if defined(LWS_WITH_SECURE_STREAMS)
 	if (wsi->for_ss) {
 
@@ -220,7 +232,7 @@ __lws_free_wsi(struct lws *wsi)
 			lws_sspc_handle_t *h = (lws_sspc_handle_t *)
 							wsi->a.opaque_user_data;
 			if (h) {
-				h->cwsi = NULL;
+				h->txp_path.priv_onw = NULL;
 				wsi->a.opaque_user_data = NULL;
 			}
 		} else
@@ -348,6 +360,9 @@ __lws_close_free_wsi(struct lws *wsi, enum lws_close_status reason,
 {
 	struct lws_context_per_thread *pt;
 	const struct lws_protocols *pro;
+#if defined(LWS_WITH_SECURE_STREAMS)
+	lws_ss_handle_t *hh = NULL;
+#endif
 	struct lws_context *context;
 	struct lws *wsi1, *wsi2;
 	int n, ccb;
@@ -383,8 +398,14 @@ __lws_close_free_wsi(struct lws *wsi, enum lws_close_status reason,
 #endif
 
 #if defined(LWS_WITH_SYS_ASYNC_DNS)
-	if (wsi == context->async_dns.wsi)
-		context->async_dns.wsi = NULL;
+	/* is this wsi handling the interface to a dns server? */
+	{
+		lws_async_dns_server_t *dsrv =
+			__lws_async_dns_server_find_wsi(&context->async_dns, wsi);
+
+		if (dsrv)
+			dsrv->wsi = NULL;
+	}
 #endif
 
 	lws_pt_assert_lock_held(pt);
@@ -571,6 +592,9 @@ just_kill_connection:
 #endif
 
 	lws_sul_cancel(&wsi->sul_connect_timeout);
+#if defined(WIN32)
+	lws_sul_cancel(&wsi->win32_sul_connect_async_check);
+#endif
 #if defined(LWS_WITH_SYS_ASYNC_DNS)
 	lws_async_dns_cancel(wsi);
 #endif
@@ -629,7 +653,7 @@ just_kill_connection:
 		wsi->socket_is_permanently_unusable = 1;
 
 		lws_inform_client_conn_fail(wsi,
-			(void *)_reason, sizeof(_reason));
+			(void *)_reason, sizeof(_reason) - 1);
 	}
 #endif
 
@@ -655,6 +679,10 @@ just_kill_connection:
 			case LWS_SSL_CAPABLE_MORE_SERVICE_READ:
 			case LWS_SSL_CAPABLE_MORE_SERVICE_WRITE:
 			case LWS_SSL_CAPABLE_MORE_SERVICE:
+				if (wsi->lsp_channel++ == 8) {
+					lwsl_wsi_info(wsi, "avoiding shutdown spin");
+					lwsi_set_state(wsi, LRS_SHUTDOWN);
+				}
 				break;
 			}
 		} else
@@ -786,7 +814,7 @@ just_kill_connection:
 		if (!wsi->a.protocol && wsi->a.vhost && wsi->a.vhost->protocols)
 			pro = &wsi->a.vhost->protocols[0];
 
-		if (pro)
+		if (pro && pro->callback && wsi->role_ops)
 			pro->callback(wsi,
 				wsi->role_ops->close_cb[lwsi_role_server(wsi)],
 				wsi->user_space, NULL, 0);
@@ -829,31 +857,24 @@ async_close:
 					lws_metrics_caliper_report_hist(h->cal_txn, (struct lws *)NULL);
 #endif
 
-					h->cwsi = NULL;
+					h->txp_path.priv_onw = NULL;
 					//wsi->a.opaque_user_data = NULL;
 				}
 			} else
 #endif
 		{
-			lws_ss_handle_t *h = (lws_ss_handle_t *)wsi->a.opaque_user_data;
+			hh = (lws_ss_handle_t *)wsi->a.opaque_user_data;
 
-			if (h) { // && (h->info.flags & LWSSSINFLAGS_ACCEPTED)) {
+			if (hh) { // && (h->info.flags & LWSSSINFLAGS_ACCEPTED)) {
 
 				/*
 				 * ss level: only reports if dangling caliper
 				 * not already reported
 				 */
-				lws_metrics_caliper_report_hist(h->cal_txn, wsi);
+				lws_metrics_caliper_report_hist(hh->cal_txn, wsi);
 
-				h->wsi = NULL;
+				hh->wsi = NULL;
 				wsi->a.opaque_user_data = NULL;
-
-				if (h->ss_dangling_connected &&
-				    lws_ss_event_helper(h, LWSSSCS_DISCONNECTED) ==
-						    LWSSSSRET_DESTROY_ME) {
-
-					lws_ss_destroy(&h);
-				}
 			}
 		}
 	}
@@ -868,6 +889,12 @@ async_close:
 			return;
 
 	__lws_close_free_wsi_final(wsi);
+
+#if defined(LWS_WITH_SECURE_STREAMS)
+	if (hh && hh->ss_dangling_connected &&
+	    lws_ss_event_helper(hh, LWSSSCS_DISCONNECTED) == LWSSSSRET_DESTROY_ME)
+		lws_ss_destroy(&hh);
+#endif
 }
 
 
@@ -915,7 +942,11 @@ __lws_close_free_wsi_final(struct lws *wsi)
 		if (pt->pipe_wsi == wsi)
 			pt->pipe_wsi = NULL;
 		if (pt->dummy_pipe_fds[0] == wsi->desc.sockfd)
+               {
+#if !defined(LWS_PLAT_FREERTOS)
 			pt->dummy_pipe_fds[0] = LWS_SOCK_INVALID;
+#endif
+               }
 	}
 
 	wsi->desc.sockfd = LWS_SOCK_INVALID;
