@@ -410,7 +410,7 @@ _lws_struct_sq3_ser_one(sqlite3 *pdb, const lws_struct_map_t *schema,
 		did++;
 		if (did != nef) {
 			if (sql_est - (unsigned int)m < 6)
-				return -1;
+				goto bail;
 			sql[m++] = ',';
 			sql[m++] = ' ';
 		}
@@ -421,6 +421,7 @@ _lws_struct_sq3_ser_one(sqlite3 *pdb, const lws_struct_map_t *schema,
 	n = sqlite3_exec(pdb, sql, NULL, NULL, NULL);
 	if (n != SQLITE_OK) {
 		lwsl_err("%s\n", sql);
+bail:
 		free(sql);
 		lwsl_err("%s: %s: fail\n", __func__, sqlite3_errmsg(pdb));
 		return -1;
@@ -524,7 +525,8 @@ lws_struct_sq3_open(struct lws_context *context, const char *sqlite3_path,
 	if (uid)
 		if (chown(sqlite3_path, uid, gid))
 			lwsl_err("%s: failed to chown %s\n", __func__, sqlite3_path);
-	chmod(sqlite3_path, 0600);
+	if (chmod(sqlite3_path, 0600))
+		lwsl_err("%s: failed to chmod %s\n", __func__, sqlite3_path);
 
 	lwsl_debug("%s: created %s owned by %u:%u mode 0600\n", __func__,
 			sqlite3_path, (unsigned int)uid, (unsigned int)gid);
