@@ -42,7 +42,7 @@
 #include "private-lib-core.h"
 
 #include "esp_system.h"
-#include "esp_spi_flash.h"
+#include "spi_flash_mmap.h"
 #include "esp_wifi.h"
 #include <nvs_flash.h>
 #include <esp_netif.h>
@@ -406,6 +406,8 @@ lws_netdev_plat_wifi_init(void)
 		return 1;
 	}
 
+	 ESP_ERROR_CHECK(esp_wifi_set_mode(WIFI_MODE_STA));
+
 	return 0;
 }
 
@@ -445,11 +447,11 @@ lws_netdev_wifi_up_plat(struct lws_netdev_instance *nd)
 		return 0;
 
 	ESP_ERROR_CHECK(esp_event_handler_instance_register(IP_EVENT,
-			  IP_EVENT_STA_GOT_IP, &_event_handler_ip, nd,
+			  IP_EVENT_STA_GOT_IP, _event_handler_ip, nd,
 			  &wnde32->instance_got_ip));
 
 	ESP_ERROR_CHECK(esp_event_handler_instance_register(WIFI_EVENT,
-			  ESP_EVENT_ANY_ID, &_event_handler_wifi, nd,
+			  ESP_EVENT_ANY_ID, _event_handler_wifi, nd,
 			  &wnde32->instance_any_id));
 
 	esp_wifi_start();
