@@ -178,7 +178,24 @@ lws_plat_init(struct lws_context *context,
 		return 1;
 	}
 
-#if defined(LWS_WITH_PLUGINS)
+#if defined(LWS_HAVE_SSL_CTX_set_keylog_callback) && !defined(__COVERITY__) && \
+		defined(LWS_WITH_TLS) && defined(LWS_WITH_CLIENT)
+	{
+		char *klf_env = getenv("SSLKEYLOGFILE");
+		size_t n = 0;
+
+		/* ... coverity taint with lws_strncpy()... */
+
+		while (klf_env && n < sizeof(context->keylog_file) - 1 &&
+		       klf_env[n]) {
+			context->keylog_file[n] = klf_env[n];
+			n++;
+		}
+		context->keylog_file[n] = '\0';
+	}
+#endif
+
+#if defined(LWS_WITH_PLUGINS) && !defined(LWS_WITH_PLUGINS_BUILTIN)
 	{
 		char *ld_env = getenv("LD_LIBRARY_PATH");
 
@@ -196,6 +213,11 @@ lws_plat_init(struct lws_context *context,
 					 "lws_protocol_plugin", NULL,
 					 protocol_plugin_cb, context);
 	}
+
+#endif
+#if defined(LWS_BUILTIN_PLUGIN_NAMES) && defined(LWS_WITH_PLUGINS)
+	lws_plugins_handle_builtin(&context->plugin_list,
+				   protocol_plugin_cb, context);
 #endif
 
 
