@@ -25,6 +25,62 @@
 #include "private-lib-core.h"
 #include "private-lib-tls.h"
 
+#if defined(LWS_HAVE_SSL_CTX_set_keylog_callback) && defined(LWS_WITH_NETWORK) && \
+	defined(LWS_WITH_TLS) && !defined(LWS_WITH_MBEDTLS) && \
+	(!defined(LWS_WITHOUT_CLIENT) || !defined(LWS_WITHOUT_SERVER))
+void
+lws_klog_dump(const SSL *ssl, const char *line)
+{
+	struct lws *wsi = (struct lws *)SSL_get_ex_data(ssl,
+					  openssl_websocket_private_data_index);
+	char path[128], hdr[128], ts[64];
+	size_t w = 0, wx = 0;
+	int fd, t;
+
+	if (!wsi || !wsi->a.context->keylog_file[0] || !wsi->a.vhost)
+		return;
+
+	lws_snprintf(path, sizeof(path), "%s.%s", wsi->a.context->keylog_file,
+			wsi->a.vhost->name);
+
+	fd = open(path, O_CREAT | O_RDWR | O_APPEND, 0600);
+	if (fd == -1) {
+		lwsl_vhost_warn(wsi->a.vhost, "Failed to append %s", path);
+		return;
+	}
+
+	/* the first item in the chunk */
+	if (!strncmp(line, "SERVER_HANDSHAKE_TRAFFIC_SECRET", 31)) {
+		w += (size_t)write(fd, "\n# ", 3);
+		wx += 3;
+		t = lwsl_timestamp(LLL_WARN, ts, sizeof(ts));
+		wx += (size_t)t;
+		w += (size_t)write(fd, ts, (size_t)t);
+
+		t = lws_snprintf(hdr, sizeof(hdr), "%s\n", wsi->lc.gutag);
+		w += (size_t)write(fd, hdr, (size_t)t);
+		wx += (size_t)t;
+
+		lwsl_vhost_warn(wsi->a.vhost, "appended ssl keylog: %s", path);
+	}
+
+	wx += strlen(line) + 1;
+	w += (size_t)write(fd, line, 
+#if defined(WIN32)
+			(unsigned int)
+#endif
+			strlen(line));
+	w += (size_t)write(fd, "\n", 1);
+	close(fd);
+
+	if (w != wx) {
+		lwsl_vhost_warn(wsi->a.vhost, "Failed to write %s", path);
+		return;
+	}
+}
+#endif
+
+
 #if defined(LWS_WITH_NETWORK)
 #if defined(LWS_WITH_MBEDTLS) || (defined(OPENSSL_VERSION_NUMBER) && \
 				  OPENSSL_VERSION_NUMBER >= 0x10002000L)
@@ -95,12 +151,12 @@ lws_tls_restrict_borrow(struct lws *wsi)
 static void
 _lws_tls_restrict_return(struct lws *wsi)
 {
+#if defined(LWS_WITH_SERVER)
 	struct lws_context *cx = wsi->a.context;
 
 	assert(cx->simultaneous_ssl_handshake >= 0);
 	assert(cx->simultaneous_ssl >= 0);
 
-#if defined(LWS_WITH_SERVER)
 	lws_gate_accepts(cx,
 			(cx->simultaneous_ssl_restriction &&
 			 cx->simultaneous_ssl == cx->simultaneous_ssl_restriction) ||
@@ -361,7 +417,7 @@ lws_tls_alloc_pem_to_der_file(struct lws_context *context, const char *filename,
 
 	if (!filename) {
 		/* we don't know if it's in const memory... alloc the output */
-		pem = lws_malloc(((size_t)inlen * 3) / 4, "alloc_der");
+		pem = lws_malloc(((size_t)(inlen + 3) * 3) / 4, "alloc_der");
 		if (!pem) {
 			lwsl_err("a\n");
 			return 1;
@@ -377,7 +433,6 @@ lws_tls_alloc_pem_to_der_file(struct lws_context *context, const char *filename,
 		p++;
 
 	if (*p != '-') {
-		lwsl_err("b\n");
 		goto bail;
 	}
 
@@ -385,7 +440,6 @@ lws_tls_alloc_pem_to_der_file(struct lws_context *context, const char *filename,
 		p++;
 
 	if (p >= end) {
-		lwsl_err("c\n");
 		goto bail;
 	}
 
@@ -398,10 +452,8 @@ lws_tls_alloc_pem_to_der_file(struct lws_context *context, const char *filename,
 	while (q > opem && *q != '\n')
 		q--;
 
-	if (*q != '\n') {
-		lwsl_err("d\n");
+	if (*q != '\n')
 		goto bail;
-	}
 
 	/* we can't write into the input buffer for mem, since it may be in RO
 	 * const segment
@@ -409,8 +461,18 @@ lws_tls_alloc_pem_to_der_file(struct lws_context *context, const char *filename,
 	if (filename)
 		*q = '\0';
 
-	*amount = (unsigned int)lws_b64_decode_string_len((char *)p, lws_ptr_diff(q, p),
-					    (char *)pem, (int)(long long)len);
+	n = lws_ptr_diff(q, p);
+	if (n == -1) /* coverity */
+		goto bail;
+
+	n = lws_b64_decode_string_len((char *)p, n,
+				      (char *)pem, (int)(long long)len);
+	if (n < 0) {
+		lwsl_err("%s: base64 pem decode failed\n", __func__);
+		goto bail;
+	}
+
+	*amount = (unsigned int)n;
 	*buf = (uint8_t *)pem;
 
 	return 0;
